function [avr] = read_besa_avr(filename)

% READ_BESA_AVR reads average EEG data in BESA format
%
% Use as
%   [avr] = read_besa_avr(filename)
%
% This will return a structure with the header information in
%   avr.npnt
%   avr.tsb
%   avr.di
%   avr.sb
%   avr.sc
%   avr.Nchan   (optional)
%   avr.label   (optional)
% and the ERP data is contained in the Nchan X Nsamples matrix
%   avr.data

% Copyright (C) 2003-2005, Robert Oostenveld
% 
% $Log: read_besa_avr.m,v $
% Revision 1.2  2005/04/25 11:10:27  roboos
% changed output labels into cell-array
% added support for reading channel labels from accompanying elp file
%
% Revision 1.1  2005/03/31 07:09:37  roboos
% old implementation, but new implementation of support for avr files that contain channel names
%

fid = fopen(filename, 'rt');

try
  % the first line contains header information
  buf = fscanf(fid, 'Npts= %d TSB= %f DI= %f  SB= %f SC= %f Nchan= %f\n');
  avr.npnt  = buf(1);
  avr.tsb   = buf(2);
  avr.di    = buf(3);
  avr.sb    = buf(4);
  avr.sc    = buf(5);
  avr.Nchan = buf(6);
  % the second line contains channel names
  chanstring = fgetl(fid);
  avr.label  = str2cell(strrep(deblank(chanstring), '''', ''))';
catch
  % rewind to the beginning of the file
  fseek(fid, 0, 'bof');
  % the first line contains header information, there is no channel information
  buf = fscanf(fid, 'Npts= %d TSB= %f DI= %f  SB= %f SC= %f\n');
  avr.npnt  = buf(1);
  avr.tsb   = buf(2);
  avr.di    = buf(3);
  avr.sb    = buf(4);
  avr.sc    = buf(5);
  % try to obtain the channel names
  [root , name]=fileparts(filename);
  elpfile = fullfile(root,[ name '.elp']);
  if exist(elpfile, 'file')
    % read the channel names from the accompanying ELP file
    lbl = importdata(elpfile);
    avr.label = strrep(lbl.textdata(:,2) ,'''', '');
  else
    warning('Could not create channels labels.');
  end
end

buf  = fscanf(fid, '%f');
nchan = length(buf)/avr.npnt;
avr.data = reshape(buf, avr.npnt, nchan)'; 
fclose(fid);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION to cut a string into pieces at the spaces
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function c = str2cell(s)
c = {};
[t, r] = strtok(s, ' ');
while ~isempty(t)
  c{end+1} = t;
  [t, r] = strtok(r, ' ');
end 
